#include <P18F452.inc>

	extern VisualInitialization

; 
ON_THRESHOLD	equ	0x75
PWM_CYCLE		equ .30
FLASH_CYCLE 	equ .2
PWM0_DEF		equ .10
PWM1_DEF		equ	.30
PWM2_DEF		equ .10
TURN_DIFF		equ .5		;     
TURN_INIT_COUNT	equ	.10		; 10   ,   :)
TURNING_LEFT	equ	.0		;  ( 0  TURNING)
TURNING_RIGHT	equ	.1		;  ( 1  TURNING)
TURN_COUNT		equ .20
BRAKE_COUNT		equ .50
BRAKE_DIFF		equ	.5

; PORTC 
PORTC_LIGHT		equ .0
PORTC_TURNLEFT	equ .2
PORTC_TURNRIGHT	equ .1
PORTC_BRAKE		equ .3

; 
	UDATA
;       
W_TEMP				res 1
STATUS_TEMP			res 1
BSR_TEMP			res 1
	
CHANNEL				res	4	;   
CHANNEL_MASK 		res 1	;  
CHANNEL_MASK_TEMP	res 1	;  

; PWM     
PWM0				res 1
PWM0_MASK			res 1
PWM0_COUNTER		res 1
PWM1				res 1
PWM1_MASK			res 1
PWM1_COUNTER		res 1
PWM2				res 1
PWM2_MASK			res 1
PWM2_COUNTER		res 1
PWM_CYCLE_COUNTER	res 1

FLASH_CYCLE_COUNTER	res 1
FLASH_STATE			res 1
FLASHING			res 1

;   2-  ()   
TURN_CENTER			res 1
TURN_CENTER_CHECK	res 1
TURN_INIT_COUNTER	res 1
TURN_COUNTER_L		res 1
TURN_COUNTER		res 1
TURNING				res 1

BRAKE_COUNTER		res 1
BRAKE_MAX			res 1
BRAKE_CUR			res 1
BRAKING				res 1

; 0 -  PWM0,  PWM1,  PWM2
; 1 -  PWM1,  PWM2,  PWM0
;d1	d2	i3
;d1 >> 3
;	d2	i3	i1
;d1 << 2, d2 >> 1, d3 >> 1
;	d1	d2	i3
;

; 
RST		CODE	0x00
	goto	START

INTR	CODE	0x08
	goto	INTERRUPTS

;-------------------------
; MAIN
;-------------------------
MAIN	CODE
START:
	call	VisualInitialization
	
	;  
	clrf	CHANNEL
	clrf	CHANNEL + 1
	clrf	CHANNEL + 2
	clrf 	CHANNEL + 3
	movlw	0x10
	movwf	CHANNEL_MASK ;   
	
	movlw	PWM0_DEF
	movwf	PWM0
	movlw	.1
	movwf	PWM0_MASK
	clrf	PWM0_COUNTER

	movlw	PWM1_DEF
	movwf	PWM1
	movlw	.2
	movwf	PWM1_MASK
	clrf	PWM1_COUNTER

	movlw	PWM2_DEF
	movwf	PWM2
	movlw	.4
	movwf	PWM2_MASK
	clrf	PWM2_COUNTER

	movlw	PWM_CYCLE
	movwf	PWM_CYCLE_COUNTER
	
	movlw	FLASH_CYCLE
	movwf	FLASH_CYCLE_COUNTER
	clrf	FLASH_STATE
	clrf	FLASHING

	;  
	clrf	TURN_CENTER
	movlw	TURN_INIT_COUNT
	movwf	TURN_INIT_COUNTER
	clrf	TURNING
	movlw	TURN_COUNT
	movwf	TURN_COUNTER

	;  
	clrf	BRAKE_COUNTER
	clrf	BRAKE_MAX
	clrf	BRAKING

	;   
	bsf		T1CON, 	TMR1ON
	bsf		T2CON,	TMR2ON
	
	; 
	;    B
	bcf		INTCON,	RBIF
	bsf		INTCON,	RBIE
	;  1
	bcf		PIR1,	TMR1IF
	bsf		PIE1,	TMR1IE
	;  2
	bcf		PIR1,	TMR2IF
	bsf		PIE1,	TMR2IE
	
	
	bsf		INTCON, GIEL
	bsf		INTCON, GIE

	;  
MAIN_LOOP:

LIGHTS:
	; ( 4)
	movlw	ON_THRESHOLD
	subwf	CHANNEL + 3, W
	btfsc	STATUS, C
	goto	LIGHTS_ON
	bcf		PORTC, PORTC_LIGHT
	goto	LIGHTS_END
LIGHTS_ON:
	bsf		PORTC, PORTC_LIGHT
LIGHTS_END:
	
	;  ( 3)
	movlw	ON_THRESHOLD
	subwf	CHANNEL + 2, W
	btfsc	STATUS, C
	goto	FLASHING_ON
	bcf		FLASHING, 0
	clrf	PORTD
	goto	FLASHING_END
FLASHING_ON:
	bsf		FLASHING, 0
FLASHING_END:

	; 

	;  
TURN_INIT:
	movf	TURN_INIT_COUNTER, F
	bz		TURN_INIT_END
	movf	CHANNEL + 1, F
	bz		TURN_END
	decf	TURN_INIT_COUNTER
	bnz		TURN_END
	movff	CHANNEL + 1, TURN_CENTER
TURN_INIT_END:

TURN:
	movf	TURN_CENTER, W
	subwf	CHANNEL + 1, W
	bc		TURN_RIGHT
TURN_LEFT:
	movwf	TURN_CENTER_CHECK
	negf	TURN_CENTER_CHECK
	movf	TURN_CENTER_CHECK, W
	sublw	TURN_DIFF
	bc		TURN_OFF
	bcf		TURNING, TURNING_RIGHT
	bcf		PORTC, 	PORTC_TURNRIGHT
	bsf		TURNING, TURNING_LEFT
	goto	TURN_END
TURN_RIGHT:
	sublw	TURN_DIFF
	bc		TURN_OFF
	bcf		TURNING, TURNING_LEFT
	bcf		PORTC, 	PORTC_TURNLEFT
	bsf		TURNING, TURNING_RIGHT
	goto	TURN_END
TURN_OFF:
	clrf	TURNING
	bcf		PORTC, PORTC_TURNLEFT
	bcf		PORTC, PORTC_TURNRIGHT
	
TURN_END:

	; 
BRAKE:
	movff	CHANNEL + 0, BRAKE_CUR
	movf	BRAKE_MAX, W
	subwf	BRAKE_CUR, W
	bnc		BRAKE_CHECK
	movff	BRAKE_CUR, BRAKE_MAX
	goto	BRAKE_END
BRAKE_CHECK:
	addlw	BRAKE_DIFF
	bc		BRAKE_END
	;  !
	bsf		PORTC, PORTC_BRAKE
	movlw	BRAKE_COUNT
	movwf	BRAKE_COUNTER
	movff	BRAKE_CUR, BRAKE_MAX
BRAKE_END:

	goto	MAIN_LOOP


;-------------------------
;  
;-------------------------
INTERRUPTS CODE

INTERRUPTS:
;   
	movwf	W_TEMP
	movff	STATUS, STATUS_TEMP
	movff	BSR, BSR_TEMP

;   
	btfsc	INTCON, RBIF
	call	RB_INTERRUPT
	btfsc	PIR1, TMR1IF
	call	TMR1_INTERRUPT
	btfsc	PIR1, TMR2IF
	call	TMR2_INTERRUPT

;    
	movff	BSR_TEMP, BSR
	movf	W_TEMP, W	
	movff	STATUS_TEMP, STATUS

	retfie

;-------------------------
;    RB
;-------------------------
RB_INTERRUPT:
	movf	PORTB, W
	movf	CHANNEL_MASK, W
	andwf	PORTB, W
	bz		RB_CHECKEND
RB_CHECKSTART:
	btfsc	T0CON,	TMR0ON
	goto	RB_END	;  
	; 
	clrf	TMR0L	;   
	bsf		T0CON, TMR0ON
	goto	RB_END

RB_CHECKEND:
	btfss	T0CON, TMR0ON
	goto	RB_END	;    
	; 
	bcf		T0CON, TMR0ON
	call	SAVE_CHANNEL
	call	NEXT_CHANNEL
	;  TMR1,    
	clrf	TMR1H
	clrf	TMR1L
RB_END:
	bcf		INTCON, RBIF
	return

;   ( TMR0)  
SAVE_CHANNEL:
	lfsr	FSR0, CHANNEL
	movff	CHANNEL_MASK, CHANNEL_MASK_TEMP
SAVE_LOOP:
	btfsc	CHANNEL_MASK_TEMP, 7
	goto	SAVE_ENDLOOP
	rlncf	CHANNEL_MASK_TEMP, F
	movf	POSTINC0, F				;  
	goto	SAVE_LOOP	
SAVE_ENDLOOP:
	movff	TMR0L, POSTINC0
	return	
	

;-------------------------
;   TMR1 - 36 
;   
;-------------------------
TMR1_INTERRUPT:
	;   0
	bcf		T0CON,	TMR0ON
	;    
	call	NEXT_CHANNEL

	; 
	bcf		PIR1,	TMR1IF
	return

NEXT_CHANNEL:
	;   
	rlncf	CHANNEL_MASK, F
	btfss	CHANNEL_MASK, 0	; ,     0
	return
	movlw	0x10
	movwf	CHANNEL_MASK ;    
	return		


;-------------------------
;   TMR2 -  19 
; 
;-------------------------
TMR2_INTERRUPT:
	;goto	FLASH_END
	movf	FLASHING, F
	bz		FLASH_END

	;       PWM 
	movf	PORTD, W

	;PWM0
	movf	PWM0_COUNTER, F
	bz		PWM0_OFF
	decf	PWM0_COUNTER, F
	iorwf	PWM0_MASK, W
	goto	PWM0_END
PWM0_OFF:
	comf	PWM0_MASK
	andwf	PWM0_MASK, W
	comf	PWM0_MASK
PWM0_END:

	;PWM1
	movf	PWM1_COUNTER, F
	bz		PWM1_OFF
	decf	PWM1_COUNTER, F
	iorwf	PWM1_MASK, W
	goto	PWM1_END
PWM1_OFF:
	comf	PWM1_MASK
	andwf	PWM1_MASK, W
	comf	PWM1_MASK
PWM1_END:

	;PWM2
	movf	PWM2_COUNTER, F
	bz		PWM2_OFF
	decf	PWM2_COUNTER, F
	iorwf	PWM2_MASK, W
	goto	PWM2_END
PWM2_OFF:
	comf	PWM2_MASK
	andwf	PWM2_MASK, W
	comf	PWM2_MASK
PWM2_END:
	
	movwf	PORTD

	;  
	decf	PWM_CYCLE_COUNTER
	bnz		PWM_END
	; 
	movlw	PWM_CYCLE
	movwf	PWM_CYCLE_COUNTER
	movff	PWM0, PWM0_COUNTER
	movff	PWM1, PWM1_COUNTER
	movff	PWM2, PWM2_COUNTER


FLASH:
	decf	FLASH_CYCLE_COUNTER
	bnz		FLASH_END
	movlw	FLASH_CYCLE
	movwf	FLASH_CYCLE_COUNTER

	movf	FLASH_STATE
	bnz		FLASH1
;  0
FLASH0:
	decf	PWM1, F
	incf	PWM2, F
	decf	PWM0, F
	bnz		FLASH_END
	;   1
	rlncf	PWM0_MASK
	rlncf	PWM0_MASK
	rlncf	PWM0_MASK
	bsf		FLASH_STATE, 0
	clrf	PORTD
	goto	FLASH_END

;  1
FLASH1:
	incf	PWM2, F
	incf	PWM0, F
	decf	PWM1, F
	movlw	PWM0_DEF
	subwf	PWM1, W
	bnz		FLASH_END
	;    0
	rrncf	PWM0_MASK
	rrncf	PWM0_MASK
	rlncf	PWM1_MASK
	rlncf	PWM2_MASK
	bcf		FLASH_STATE, 0
	movlw	PWM0_DEF
	movwf	PWM0
	movlw	PWM1_DEF
	movwf	PWM1
	movlw	PWM2_DEF
	movwf	PWM2
	clrf	PORTD
	goto 	FLASH_END

FLASH_END:
PWM_END:

ITURN:
	decf	TURN_COUNTER_L
	bnz		ITURN_END
	call	IBRAKE
	decf	TURN_COUNTER
	bnz		ITURN_END
	movlw	TURN_COUNT
	movwf	TURN_COUNTER

	btfsc	TURNING, TURNING_LEFT
	btg		PORTC, PORTC_TURNLEFT
	btfsc	TURNING, TURNING_RIGHT
	btg		PORTC, PORTC_TURNRIGHT
ITURN_END:

	; 
	bcf		PIR1,	TMR2IF
	return


IBRAKE:
	movf	BRAKE_COUNTER, F
	bz		IBRAKE_END
	decf	BRAKE_COUNTER
	bnz		IBRAKE_END
	bcf		PORTC, PORTC_BRAKE

IBRAKE_END:
	return

	
	END